exact_fetched = {
    "aerospike": "https://github.com/aerospike/aerospike-client-python",
    "appier": "https://github.com/hivesolutions/appier",
    "appium-python-client": "https://github.com/appium/python-client",
    "boxsdk": "https://github.com/box/box-python-sdk",
    "breezy": "https://github.com/breezy-team/breezy",  # should be https://launchpad.net/brz, but that has a higher distance
    "django-lfs": "https://github.com/diefenbach/django-lfs",
    "giacpy": "https://gitlab.math.univ-paris-diderot.fr/han/giacpy",
    "github-api-python": "https://github.com/hivesolutions/github_api",
    "libvirt-python": "https://gitlab.com/libvirt/libvirt-python",
    "mygpoclient": "https://github.com/gpodder/mygpoclient",
    "oslo-upgradecheck": "https://opendev.org/openstack/oslo.upgradecheck",
    "oslo-vmware": "https://opendev.org/openstack/oslo.vmware",
    "pycli": "https://github.com/whilp/cli",
    "pycryptodomex": "https://github.com/Legrandin/pycryptodome",
    "pynliner": "https://github.com/rennat/pynliner",
    "pyriemann": "https://github.com/alexandrebarachant/pyRiemann",
    "python-nss": "https://hg.mozilla.org/projects/python-nss",
    "python-termstyle": "https://github.com/gfxmonk/termstyle",
    "pywurfl": "https://sourceforge.net/projects/wurfl",
    "PyZ3950": "https://www.panix.com/~asl2/software/PyZ3950",
    "roundup": "https://sourceforge.net/projects/roundup",
    "rpy2": "https://github.com/rpy2/rpy2",
    "South": "https://bitbucket.org/andrewgodwin/south",
    "tinysegmenter": "http://git.tuxfamily.org/tinysegmente/tinysegmenter/",
    "webapp2": "https://github.com/GoogleCloudPlatform/webapp2",  # upgrade to https
    "web.py": "https://github.com/webpy/webpy",
}

exact_metadata = {
    "agate-excel": "https://github.com/wireservice/agate-excel",
    "alex-ber-utils": "https://github.com/alex-ber/AlexBerUtils",
    "algoliasearch": "https://github.com/algolia/algoliasearch-client-python",
    "ansible-tower-cli": "https://github.com/ansible/tower-cli",
    "apache-airflow": "https://github.com/apache/airflow",
    "apache-beam": "https://github.com/apache/beam",
    "aspy-refactor-imports": "https://github.com/asottile/aspy.refactor_imports",
    "auger-python": "https://github.com/laffra/auger",
    "backports-functools-lru-cache": "https://github.com/jaraco/backports.functools_lru_cache",
    "backports-shutil-get-terminal-size": "https://github.com/chrippa/backports.shutil_get_terminal_size",
    "backports-shutil-which": "https://github.com/minrk/backports.shutil_which",
    "backports-ssl-match-hostname": "https://bitbucket.org/brandon/backports.ssl_match_hostname",
    "betamax-serializers": "https://gitlab.com/betamax/serializers",
    "bolt-python": "https://github.com/bolt-project/bolt",
    "BytecodeAssembler": "https://github.com/peak-legacy/bytecodeassembler",
    "certbot": "https://github.com/certbot/certbot",  # TODO rename https://github.com/letsencrypt/letsencrypt
    "certifi": "https://github.com/certifi/python-certifi",
    "cffi": "https://foss.heptapod.net/pypy/cffi",  # https://foss.heptapod.net/pypy/cffi/issues/443
    "config": "https://bitbucket.org/vinay.sajip/config",
    "Contextual": "https://github.com/peak-legacy/contextual",
    "couchbase": "https://github.com/couchbase/couchbase-python-client",
    "cssutils": "https://bitbucket.org/cthedot/cssutils",
    "dbus-signature-pyparsing": "https://github.com/stratis-storage/dbus-signature-pyparsing",
    "ddtrace": "https://github.com/DataDog/dd-trace-py",
    "DecoratorTools": "https://github.com/peak-legacy/decoratortools",
    "delegator-py": "https://github.com/kennethreitz/delegator",
    "dephell-pythons": "https://github.com/dephell/dephell_pythons",
    "dephell-shells": "https://github.com/dephell/dephell_shells",
    "dephell-specifier": "https://github.com/dephell/dephell_specifier",
    "django-gravatar": "https://code.google.com/p/django-gravatar",
    "django-hijack-admin": "https://github.com/arteria/django-hijack-admin",
    "django-pagination": "https://code.google.com/p/django-pagination",
    "djangorestframework-bulk": "https://github.com/miki725/django-rest-framework-bulk",
    "djangorestframework-csv": "https://github.com/mjumbewu/django-rest-framework-csv",
    "djangorestframework-filters": "https://github.com/philipn/django-rest-framework-filters",
    "djangorestframework-gis": "https://github.com/djangonauts/django-rest-framework-gis",
    "djangorestframework-jsonapi": "https://github.com/django-json-api/django-rest-framework-json-api",
    "djangorestframework-jsonp": "https://github.com/jpadilla/django-rest-framework-jsonp",
    "djangorestframework-jwt": "https://github.com/GetBlimp/django-rest-framework-jwt",
    "djangorestframework-oauth": "https://github.com/jpadilla/django-rest-framework-oauth",
    "djangorestframework-recursive": "https://github.com/heywbj/django-rest-framework-recursive",
    "djangorestframework-sso": "https://github.com/namespace-ee/django-rest-framework-sso",
    "djangorestframework-xml": "https://github.com/jpadilla/django-rest-framework-xml",
    "djangorestframework-yaml": "https://github.com/jpadilla/django-rest-framework-yaml",
    "django-websocket": "https://github.com/gregmuellegger/django-websocket",
    "docker-pycreds": "https://github.com/shin-/dockerpy-creds",
    "dotnetcore2": "https://github.com/dotnet/core",
    "drfdocs": "https://github.com/manosim/django-rest-framework-docs",
    "drf-jwt": "https://github.com/Styria-Digital/django-rest-framework-jwt",
    "edx-ccx-keys": "https://github.com/edx/ccx-keys",
    "edx-completion": "https://github.com/edx/completion",
    "edx-i18n-tools": "https://github.com/edx/i18n-tools",
    "edx-opaque-keys": "https://github.com/edx/opaque-keys",
    "elasticsearch2": "https://github.com/elastic/elasticsearch-py",
    "empy": "http://www.alcyone.com/software/empy",
    "etcd": "https://github.com/dsoprea/PythonEtcdClient",
    "evergreen-py": "https://github.com/evergreen-ci/evergreen.py",
    "facebookads": "https://github.com/facebook/facebook-python-ads-sdk",
    "facebook-business": "https://github.com/facebook/facebook-python-business-sdk",
    "gdata": "https://github.com/google/gdata-python-client",
    "getchanges": "https://github.com/TheKevJames/getchanges",
    "girder-client": "https://github.com/girder/girder",
    "google-apitools": "https://github.com/google/apitools",
    "google-nucleus": "https://github.com/google/nucleus",
    "google-pasta": "https://github.com/google/pasta",
    "graphql-core": "https://github.com/graphql-python/graphql-core",
    "graphql-core-next": "https://github.com/graphql-python/graphql-core-next",
    "guess_language-spirit": "https://bitbucket.org/spirit/guess_language",
    "hacs-frontend": "https://github.com/hacs/frontend",
    "hdfs": "https://github.com/mtth/hdfs",
    "honeycomb-beeline": "https://github.com/honeycombio/beeline-python",
    "hs-dbus-signature": "https://github.com/stratis-storage/hs-dbus-signature",
    "ibm-cos-sdk-core": "https://github.com/ibm/ibm-cos-sdk-python-core",
    "ibm-cos-sdk-s3transfer": "https://github.com/ibm/ibm-cos-sdk-python-s3transfer",
    "ibm-db": "https://github.com/ibmdb/python-ibmdb",
    "imagecodecs": "https://github.com/cgohlke/imagecodecs",
    "Importing": "http://svn.eby-sarna.com/Importing/",
    "imreg": "https://github.com/cgohlke/imreg",
    "infi-clickhouse-orm": "https://github.com/Infinidat/infi.clickhouse_orm",
    "j1m.sphinxautointerface": "https://bitbucket.org/j1m/sphinxautointerface",
    "jobspy": "https://github.com/josiahcarlson/jobs",
    "k5test": "https://github.com/pythongssapi/k5test",
    "lancet-ioam": "https://github.com/ioam/lancet",
    "linaro-json": "https://launchpad.net/linaro-python-json",
    "localstack-client": "https://github.com/localstack/localstack-python-client",
    "logentries-lecli": "https://github.com/logentries/lecli",
    "logging": "https://bitbucket.org/vinay.sajip/logging",
    "lzmaffi": "https://github.com/r3m0t/backports.lzma",
    "magic": "http://www.jsnp.net/code/magic.py",
    "mailman-hyperkitty": "https://gitlab.com/mailman/mailman-hyperkitty/",
    "mesh-tensorflow": "https://github.com/tensorflow/mesh",
    "mockredispy": "https://github.com/locationlabs/mockredis",
    "neo4j-driver": "https://github.com/neo4j/neo4j-python-driver",
    "nexpose": "https://github.com/rapid7/nexpose-client-python",
    "nteract-scrapbook": "https://github.com/nteract/scrapbook",
    "open3d-python": "https://github.com/intel-isl/Open3D",
    "opentracing-instrumentation": "https://github.com/uber-common/opentracing-python-instrumentation",
    "pantsbuild-pants": "https://github.com/pantsbuild/pants",
    "path-py": "https://github.com/jaraco/path",
    "PEAK": "https://github.com/peak-legacy/peak",
    "peak-rules": "https://github.com/peak-legacy/peak-rules",
    "pep8": "https://github.com/PyCQA/pep8",
    "pgc-interface": "https://github.com/UniversalDevicesInc/pgc-python-interface",
    "pipsi": "https://github.com/mitsuhiko/pipsi",
    "pipx": "https://github.com/pipxproject/pipx",
    "piwikapi": "https://github.com/piwik/piwik-python-api",
    "pkgwat-api": "https://github.com/fedora-infra/pkgwat.api",
    "PrettyTable": "https://code.google.com/p/prettytable",
    "prometheus-client": "https://github.com/prometheus/client_python",
    "prometheus_client": "https://github.com/prometheus/client_python",
    "protobuf3-to-dict": "https://github.com/kaporzhu/protobuf-to-dict",
    "ProxyTypes": "https://github.com/peak-legacy/proxytypes",
    "punch.py": "https://github.com/lgiordani/punch",
    "py3bitbucket": "https://github.com/c9s/py-bitbucket",
    "pyarrow": "https://github.com/apache/arrow",
    "pycontracts": "https://github.com/andreacensi/contracts",
    "pyenchant": "https://github.com/pyenchant/pyenchant",
    "pyethash": "https://github.com/ethereum/ethash",
    "pygresql": "https://github.com/pygresql/pygresql",
    "pyhacrf-datamade": "https://github.com/datamade/pyhacrf",
    "pynvim": "https://github.com/neovim/pynvim",
    "pyperformance": "https://github.com/python/pyperformance",
    "pyrex": "https://www.csse.canterbury.ac.nz/greg.ewing/python/Pyrex/",
    "pyspark": "https://github.com/apache/spark",
    "pytesting-utils": "https://github.com/pytesting/utils",
    "python2-hwloc": "https://gitlab.com/guystreeter/python-hwloc",
    "python2-libnuma": "https://gitlab.com/guystreeter/python-libnuma",
    "python2-pythondialog": "https://sourceforge.net/projects/pythondialog",
    "python3-hwloc": "https://gitlab.com/guystreeter/python-hwloc",
    "python3-libnuma": "https://gitlab.com/guystreeter/python-libnuma",
    "python-package-sync-tool": "https://github.com/alex-ber/PythonPackageSyncTool",
    "python-pseudorandom": "https://github.com/smathot/python-pseudorandom",
    "pyvows": "https://github.com/heynemann/pyvows",
    "pywatchman": "https://github.com/facebook/watchman",
    "recurly": "https://github.com/recurly/recurly-client-python",
    "reportportal-client": "https://github.com/reportportal/client-Python",
    "repoze.lru": "https://github.com/repoze/repoze.lru",
    "robotframework-databaselibrary": "https://github.com/franz-see/Robotframework-Database-Library",
    "robotframework-selenium2library": "https://github.com/robotframework/Selenium2Library",
    "robotframework-seleniumlibrary": "https://github.com/robotframework/SeleniumLibrary",
    "robotframework-sshlibrary": "https://github.com/robotframework/SSHLibrary",
    "rope": "https://github.com/python-rope/rope",
    "ruamel-ordereddict": "https://sourceforge.net/projects/ruamel-ordereddict",
    "ruamel-yaml-clib": "https://sourceforge.net/projects/ruamel-yaml-clib",
    "ruamel-yaml": "https://sourceforge.net/projects/ruamel-yaml",
    "rwt": "https://github.com/jaraco/rwt",
    "rx": "https://github.com/ReactiveX/RxPY",
    "ruffus": "https://github.com/cgat-developers/ruffus",
    "s3cmd": "https://github.com/s3tools/s3cmd",
    "sailthru-client": "https://github.com/sailthru/sailthru-python-client",
    "scaleapi": "https://github.com/scaleapi/scaleapi-python-client",
    "scikits-datasmooth": "https://github.com/jjstickel/scikit-datasmooth",
    "scikits-sparse": "https://github.com/scikit-sparse/scikit-sparse",
    "semantic-version": "https://github.com/rbarrois/python-semanticversion",
    "shopifyapi": "https://github.com/Shopify/shopify_python_api",
    "shrub-py": "https://github.com/evergreen-ci/shrub.py",
    "simplegeneric": "https://github.com/peak-legacy/simplegeneric",
    "smsapi-client": "https://github.com/smsapi/smsapi-python-client",
    "sseclient-py": "https://github.com/mpetazzoni/sseclient",
    "suds-jurko": "https://bitbucket.org/jurko/suds",
    "SymbolType": "http://svn.eby-sarna.com/SymbolType/",
    "SymbolType": "http://svn.eby-sarna.com/SymbolType/",
    "tableauserverclient": "https://github.com/tableau/server-client-python",
    "tap-py": "https://github.com/python-tap/tappy",
    "tensorflow-transform": "https://github.com/tensorflow/transform",
    "termstyle": "https://github.com/gfxmonk/termstyle",
    "tg.devtools": "https://github.com/TurboGears/tg2devtools",
    "threevis": "https://www.graphics.rwth-aachen.de:9000/threevis/threevis",
    "tifffile": "https://github.com/cgohlke/tifffile",
    "tlslite": "https://github.com/trevp/tlslite",
    "torchtext": "https://github.com/pytorch/text",
    "torchvision": "https://github.com/pytorch/vision",
    "treeherder-client": "https://github.com/mozilla/treeherder",
    "Trellis": "http://svn.eby-sarna.com/Trellis/",
    "twitter-ads": "https://github.com/twitterdev/twitter-python-ads-sdk",
    "ulid-py": "https://github.com/ahawker/ulid",
    "uritemplate-py": "https://github.com/python-hyper/uritemplate",
    "utmp": "https://github.com/hjacobs/utmp",
    "uwsgi": "https://github.com/unbit/uwsgi",
    "uwsgitop": "https://github.com/xrmx/uwsgitop",
    "webstack-django-jwt-auth": "https://github.com/webstack/django-jwt-auth",
    "wsgiref": "http://svn.eby-sarna.com/wsgiref/",
    "yubico-client": "https://github.com/Kami/python-yubico-client",
    "yubico": "https://github.com/Kami/python-yubico-client",
}

exact = exact_metadata.copy()
exact.update(exact_fetched)

name_mismatch_fetched = {
    "antlr-python-runtime": "https://github.com/antlr/antlr3",
    "antlr4-python2-runtime": "https://github.com/antlr/antlr4",
    "antlr4-python3-runtime": "https://github.com/antlr/antlr4",
    "atlassian-jwt-auth": "https://github.com/atlassian/asap-authentication-python",
    "atpublic": "https://gitlab.com/warsaw/public",
    "augurlib": "https://github.com/augur-ecosystem/augur",
    "avalara": "https://github.com/avadev/AvaTax-REST-V2-Python-SDK",
    "avocado-framework": "https://github.com/avocado-framework/avocado",
    "awkward": "https://github.com/scikit-hep/awkward-array",
    "aws-cdk-assets": "https://github.com/aws/aws-cdk",
    "aws-cdk-core": "https://github.com/aws/aws-cdk",
    "aws-cdk-cx-api": "https://github.com/aws/aws-cdk",
    "aws-cdk-region-info": "https://github.com/aws/aws-cdk",
    "awscli-cwlogs": "https://github.com/aws/aws-cli",
    "aws-kinesis-agg": "https://github.com/awslabs/kinesis-aggregation",
    "aws-sam-translator": "https://github.com/awslabs/serverless-application-model",
    "azure": "https://github.com/Azure/azure-sdk-for-python",
    "barnum": "https://github.com/chris1610/barnum-proj",
    "bert-serving-server": "https://github.com/hanxiao/bert-as-service",
    "blurb": "https://github.com/python/core-workflow",
    "boto3-stubs": "https://github.com/vemel/mypy_boto3_builder",
    "buildbot-slave": "https://github.com/buildbot/buildbot",
    "buildbot-worker": "https://github.com/buildbot/buildbot",
    "c7n": "https://github.com/cloud-custodian/cloud-custodian",
    "category-encoders": "https://github.com/scikit-learn-contrib/categorical-encoding",
    "cauldron-notebook": "https://github.com/sernst/cauldron",
    "celery-redbeat": "https://github.com/sibson/redbeat",
    "cfn-flip": "https://github.com/awslabs/aws-cfn-template-flip",
    "cfscrape": "https://github.com/Anorov/cloudflare-scrape",
    "chart-studio": "https://github.com/plotly/plotly.py",
    "cmreshandler": "https://github.com/cmanaha/python-elasticsearch-logger",
    "cogapp": "https://github.com/nedbat/cog",
    "cupy-cuda100": "https://github.com/cupy/cupy",
    "datanommer-consumer": "https://github.com/fedora-infra/datanommer",
    "datanommer-models": "https://github.com/fedora-infra/datanommer",
    "ddg3": "https://github.com/jpetrucciani/python-duckduckgo",
    "doc-warden": "https://github.com/Azure/azure-sdk-tools",
    "dogpile": "https://bitbucket.org/zzzeek/dogpile.core",
    "dwave-qbsolv": "https://github.com/dwavesystems/qbsolv",
    "ebcdic": "https://github.com/roskakori/CodecMapper",
    "exifread": "https://github.com/ianare/exif-py",
    "extract-msg": "https://github.com/mattgwwalker/msg-extractor",
    "eyes-selenium": "https://github.com/applitools/eyes.sdk.python",
    "f5-sdk": "https://github.com/F5Networks/f5-common-python",
    "feather-format": "https://github.com/wesm/feather",
    "flask-opentracing": "https://github.com/opentracing-contrib/python-flask",
    "freecell_solver": "https://github.com/shlomif/fc-solve",
    "gapic-google-cloud-logging-v2": "https://github.com/googleapis/googleapis",
    "gax-google-logging-v2": "https://github.com/googleapis/googleapis",
    "gax-google-pubsub-v1": "https://github.com/googleapis/googleapis",
    "gfapi": "https://github.com/gluster/libgfapi-python",
    "google-api-core": "https://github.com/googleapis/google-cloud-python",
    "googleappenginecloudstorageclient": "https://github.com/GoogleCloudPlatform/appengine-gcs-client",
    "gviz-api": "https://github.com/google/google-visualization-python",
    "h2": "https://github.com/python-hyper/hyper-h2",
    "hpsklearn": "https://github.com/hyperopt/hyperopt-sklearn",
    "ibm-cloud-sdk-core": "https://github.com/IBM/python-sdk-core",
    "ibm-watson": "https://github.com/watson-developer-cloud/python-sdk",
    "igor": "http://git.tremily.us/?p=igor.git",
    "jdatetime": "https://github.com/slashmili/python-jalali",
    "keras-mxnet": "https://github.com/awslabs/keras-apache-mxnet",
    "lftools": "https://gerrit.linuxfoundation.org/infra/q/project:releng%252Flftools",
    "lit": "https://github.com/llvm/llvm-project",
    "locustio": "https://github.com/locustio/locust",
    "lomond": "https://github.com/wildfoundry/dataplicity-lomond",
    "mailjet-rest": "https://github.com/mailjet/mailjet-apiv3-python",
    "marionette-driver": "https://wiki.mozilla.org/Auto-tools/Projects/Marionette",
    "matrix-client": "https://github.com/matrix-org/matrix-python-sdk",
    "maxminddb": "https://github.com/maxmind/MaxMind-DB-Reader-python",
    "mercurial": "https://www.mercurial-scm.org/repo/hg",
    "mibian": "https://github.com/yassinemaaroufi/MibianLib",
    "minimal-snowplow-tracker": "https://github.com/fishtown-analytics/snowplow-python-tracker",
    "mozcrash": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozdebug": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozdevice": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozfile": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozinfo": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozinstall": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozleak": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozlog": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "moznetwork": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozprocess": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozprofile": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozrunner": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "mozversion": "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    "msal": "https://github.com/AzureAD/microsoft-authentication-library-for-python",
    "mwlib.docbook": "https://github.com/pediapress/mwlib",
    "mwlib.ext": "https://github.com/pediapress/mwlib",
    "mwlib.xhtml": "https://github.com/pediapress/mwlib",
    "mws": "https://github.com/python-amazon-mws/python-amazon-mws",
    "mypy-boto3": "https://github.com/vemel/mypy_boto3_builder",
    "nox-automation": "https://github.com/theacodes/nox",
    "opencensus-ext-logging": "https://github.com/census-instrumentation/opencensus-python",
    "opencensus-ext-requests": "https://github.com/census-instrumentation/opencensus-python",
    "opencensus-ext-stackdriver": "https://github.com/census-instrumentation/opencensus-python",
    "opencensus-ext-threading": "https://github.com/census-instrumentation/opencensus-python",
    "opencv-contrib-python-headless": "https://github.com/skvark/opencv-python",
    "opencv-contrib-python": "https://github.com/skvark/opencv-python",
    "opencv-python-headless": "https://github.com/skvark/opencv-python",
    "oss2": "https://github.com/aliyun/aliyun-oss-python-sdk",
    "pathlib-mate": "https://github.com/MacHu-GWU/pathlib_mate-project",
    "pathspec": "https://github.com/cpburnz/python-path-specification",
    "powerline-status": "https://github.com/powerline/powerline",
    "psycopg2-binary": "https://github.com/psycopg/psycopg2",
    "publicsuffixlist": "https://github.com/ko-zu/psl",
    "pycdio": "http://git.savannah.gnu.org/cgit/libcdio/pycdio.git",
    "pydevd-pycharm": "https://github.com/fabioz/PyDev.Debugger",
    "pyficache": "https://github.com/rocky/python-filecache",
    "pygtkhelpers": "https://bitbucket.org/aafshar/pygtkhelpers-main",
    "pygtk": "https://gitlab.gnome.org/GNOME/pygobject",
    "pyjarowinkler": "https://github.com/nap/jaro-winkler-distance",
    "pymongo": "https://github.com/mongodb/mongo-python-driver",
    "pyramid-arima": "https://github.com/tgsmith61591/pyramid",
    "pyside2": "http://code.qt.io/cgit/pyside/pyside-setup.git",  # close enough
    "pytest-profiling": "https://github.com/manahl/pytest-plugins",
    "pytest-server-fixtures": "https://github.com/manahl/pytest-plugins",
    "pytest-shutil": "https://github.com/manahl/pytest-plugins",
    "python-geoip-geolite2": "https://github.com/mitsuhiko/python-geoip",
    "python-vlc": "https://git.videolan.org/?p=vlc/bindings/python.git",
    "redfish": "https://github.com/DMTF/python-redfish-library",
    "regex": "https://bitbucket.org/mrabarnett/mrab-regex",
    "rfpimp": "https://github.com/parrt/random-forest-importances",
    "robotframework-pabot": "https://github.com/mkorpela/pabot",
    "rpi-gpio": "https://sourceforge.net/projects/raspberry-gpio-python",
    "scikit-surprise": "https://github.com/NicolasHug/Surprise",
    "skl2onnx": "https://github.com/onnx/sklearn-onnx",
    "sk-video": "https://github.com/scikit-video/scikit-video",
    "space-tracer": "https://github.com/donkirkby/live-py-plugin",
    "spark-parser": "https://github.com/rocky/python-spark",
    "strsimpy": "https://github.com/luozhouyang/python-string-similarity",
    "swapper": "https://github.com/wq/django-swappable-models",
    "tagpy": "http://git.tiker.net/?p=tagpy.git",
    "telepathy": "https://github.com/hackliff/intuition-plugins",
    "teradatasql": "https://github.com/Teradata/python-driver",
    "terminalone": "https://github.com/MediaMath/t1-python",
    "tf-estimator-nightly": "https://github.com/tensorflow/estimator",
    "tf-nightly-2-0-preview": "https://github.com/tensorflow/tensorflow",
    "tf-nightly-gpu": "https://github.com/tensorflow/tensorflow",
    "tf-nightly": "https://github.com/tensorflow/tensorflow",
    "thLib": "http://work.thaslwanter.at/thLib/html/",
    "turbomail": "https://github.com/marrow/marrow.mailer",
    "umap-learn": "https://github.com/lmcinnes/umap",
    "urbanairship": "https://github.com/urbanairship/python-library",
    "urlgrabber": "http://yum.baseurl.org/gitwebd27a.html?p=urlgrabber",
    "wandb": "https://github.com/wandb/client",
    "wxpython": "https://github.com/wxWidgets/Phoenix",
    "xkcdpass": "https://github.com/redacted/XKCD-password-generator",
    "yara": "https://github.com/mjdorma/yara-ctypes",
    "yoyo-migrations": "https://bitbucket.org/ollyc/yoyo",
    "zc.buildout": "https://github.com/buildout/buildout",
    "ZODB3": "https://github.com/zopefoundation/zodb",
}

name_mismatch_metadata = {
    "3to2": "https://bitbucket.org/amentajo/lib3to2",
    "absl-py": "https://github.com/abseil/abseil-py",
    "acme": "https://github.com/letsencrypt/letsencrypt",
    "adal": "https://github.com/AzureAD/azure-activedirectory-library-for-python",
    "adapt-parser": "https://github.com/MycroftAI/adapt",
    "adns": "https://github.com/trolldbois/python3-adns",
    "adyen": "https://github.com/Adyen/adyen-python-api-library",
    "aliyun-python-sdk-core": "https://github.com/aliyun/aliyun-openapi-python-sdk",
    "allure-behave": "https://github.com/allure-framework/allure-python",
    "allure-pytest": "https://github.com/allure-framework/allure-python",
    "allure-python-commons": "https://github.com/allure-framework/allure-python",
    "amazon-kclpy": "https://github.com/awslabs/amazon-kinesis-client-python",
    "ansicolors": "https://github.com/jonathaneunice/colors",
    "ansicolors": "https://github.com/jonathaneunice/colors",
    "antlr3-python-runtime": "https://github.com/antlr/antlr3",
    "apertium_lint": "https://sourceforge.net/projects/apertium",
    "asyncio-nats-client": "https://github.com/nats-io/nats.py",
    "asyncio-nats-streaming": "https://github.com/nats-io/stan.py",
    "authheaders": "https://github.com/ValiMail/authentication-headers",
    "authres": "https://launchpad.net/authentication-results-python",
    "aws-cdk-aws-iam": "https://github.com/aws/aws-cdk",
    "aws-cdk-aws-kms": "https://github.com/aws/aws-cdk",
    "aws-cdk-aws-s3": "https://github.com/aws/aws-cdk",
    "aws-client": "https://github.com/collectrium/col-aws-clients",
    "awsiotpythonsdk": "https://github.com/aws/aws-iot-device-sdk-python",
    "awswrangler": "https://github.com/awslabs/aws-data-wrangler",
    "azure-cosmosdb-nspkg": "https://github.com/Azure/azure-cosmosdb-python",
    "azure-cosmosdb-table": "https://github.com/Azure/azure-cosmosdb-python",
    "azure-devops": "https://github.com/Microsoft/vsts-python-api",
    "azure-kusto-data": "https://github.com/Azure/azure-kusto-python",
    "azure-kusto-ingest": "https://github.com/Azure/azure-kusto-python",
    "azure-storage-common": "https://github.com/Azure/azure-storage-python",
    "azure-storage-nspkg": "https://github.com/Azure/azure-storage-python",
    "azureml": "https://github.com/Azure/Azure-MachineLearning-ClientLibrary-Python",
    "azureml-model-management-sdk": "https://github.com/Azure/azure-ml-api-sdk-python",
    "basho-erlastic": "https://github.com/basho/python-erlastic",
    "bert-build": "https://git.cornhooves.org/build-tools/bert",
    "bert-serving-client": "https://github.com/hanxiao/bert-as-service",
    "bert-tensorflow": "https://github.com/google-research/bert",
    "BetterErrorMessages": "https://github.com/SylvainDe/DidYouMean-Python",
    "better-exceptions-fork": "https://github.com/delgan/better-exceptions",
    "bigcode-fetcher": "https://github.com/tuvistavie/bigcode-tools",
    "boa-api": "https://github.com/boalang/api-python",
    "boto3-type-annotations-with-docs": "https://github.com/alliefitter/boto3_type_annotations",
    "bugzilla": "https://github.com/gdestuynder/simple_bugzilla",  # https://github.com/bugzilla/bugzilla
    "cangjie": "https://github.com/bigdata-ustc/XKT",
    "cassandra-driver": "https://github.com/datastax/python-driver",
    "Catwalk": "https://github.com/pedersen/tgtools",
    "cdecimal": "https://www.bytereef.org/mpdecimal",
    "certbot-nginx": "https://github.com/letsencrypt/letsencrypt",
    "clickhouse-cityhash": "https://github.com/xzkostyan/python-cityhash",
    "cloudfoundry-client": "https://github.com/antechrestos/cf-python-client",
    "codeclimate-test-reporter": "https://github.com/codeclimate/python-test-reporter",
    "compatibility-lib": "https://github.com/GoogleCloudPlatform/cloud-opensource-python",
    "compressed-segmentation": "https://github.com/janelia-flyem/compressedseg",
    "copr-common": "https://pagure.io/copr/copr",
    "coreapi": "https://github.com/core-api/python-client",
    "coreapi": "https://github.com/core-api/python-client",
    "cornice-sphinx": "https://github.com/Cornices/cornice.ext.sphinx",
    "cql": "https://code.google.com/a/apache-extras.org/p/cassandra-dbapi2",
    "cqlsh": "http://git-wip-us.apache.org/repos/asf/cassandra.git",
    "crontab": "https://github.com/josiahcarlson/parse-crontab",
    "cryptography-vectors": "https://github.com/pyca/cryptography",
    "cs.urlutils": "https://bitbucket.org/cameron_simpson/css",
    "ctypeslib": "https://svn.python.org/projects/ctypes",
    "datadog-checks-base": "https://github.com/DataDog/integrations-core",
    "dbt-snowflake": "https://github.com/fishtown-analytics/dbt",
    "debian": "https://github.com/ourway/iran",
    "dedupe-hcluster": "https://github.com/datamade/hcluster",
    "dialogflow": "https://github.com/dialogflow/dialogflow-python-client-v2",
    "discover": "https://code.google.com/p/unittest-ext",
    "django-autoslug-iplweb": "https://github.com/neithere/django-autoslug",
    "django-common-helpers": "https://github.com/tivix/django-common",
    "django-cpserver": "https://hg.piranha.org.ua/cpserver",  # lost domain
    "django-daterange-filter": "https://github.com/tzulberti/django-datefilterspec",
    "django-events": "https://github.com/skyl/django-eventpickrs",
    "django-extra-fields": "https://github.com/Hipo/drf-extra-fields",
    "django-genericforeignkey": "http://tracpub.yaco.es/djangoapps/wiki/GenericForeignKey",
    "django-instant-coverage": "https://github.com/colons/instant-coverage",
    "django-kongoauth": "https://github.com/SchoolOrchestration/kongoauth",
    "django-notifications-hq": "https://github.com/django-notifications/django-notifications",
    "django-oot": "http://tracpub.yaco.es/djangoapps/wiki/OOT",
    "django-ranged-response": "https://github.com/wearespindle/django-ranged-fileresponse",
    "django-registration-redux": "https://github.com/macropin/django-registration",
    "djangorestframework-jwt-custom-user": "https://github.com/RegioHelden/django-rest-framework-jwt",
    "django-restricted-resource": "https://git.linaro.org/lava/django-restricted-resource.git",
    "django-roa": "http://code.welldev.org/django-roa/wiki/Home",
    "django-statsd-mozilla": "https://github.com/django-statsd/django-statsd",
    "django-stdfile": "https://tracpub.yaco.es/djangoapps/wiki/StdFile",
    "django-storages-redux": "https://github.com/jschneier/django-storages",
    "django-vz-wiki": "https://github.com/jobscry/vz-wiki",
    "djay": "https://github.com/aleontiev/dj",
    "dm-sonnet": "https://github.com/deepmind/sonnet",
    "dm-tree": "https://github.com/deepmind/tree",
    "dns-lexicon": "https://github.com/AnalogJ/lexicon",
    "docxtpl": "https://github.com/elapouya/python-docx-template",
    "dohq-artifactory": "https://github.com/devopshq/artifactory",
    "dopamine-rl": "https://github.com/google/dopamine",
    "dpcontracts": "https://github.com/deadpixi/contracts",
    "duo-web": "https://github.com/duosecurity/duo_python",
    "dynamodb-encryption-sdk": "https://github.com/aws/aws-dynamodb-encryption-python",
    "edgegrid-python": "https://github.com/akamai-open/AkamaiOPEN-edgegrid-python",
    "efilter": "https://github.com/google/dotty",
    "elastic-apm": "https://github.com/elastic/apm-agent-python",
    "elastic-apm": "https://github.com/elastic/apm-agent-python",
    "elasticsearch-async": "https://github.com/elastic/elasticsearch-py-async",
    "elasticsearch-curator": "https://github.com/elastic/curator",
    "elementtidy": "http://effbot.org/zone/element-tidylib.htm",
    "entropy-store": "https://github.com/fusionapp/entropy",
    "esrally": "https://github.com/elastic/rally",
    "etcd3gw": "https://github.com/dims/etcd3-gateway",
    "faiss-cpu": "https://github.com/kyamagu/faiss-wheels",
    "fake-factory": "https://github.com/joke2k/faker",
    "fbprophet": "https://github.com/facebook/prophet",
    "fb-re2": "https://github.com/facebook/pyre2",
    "Flask-Security-Too": "https://github.com/jwag956/flask-security",
    "flatten-json": "https://github.com/amirziai/flatten",
    "flit-core": "https://github.com/takluyver/flit",
    "fluidity-sm": "https://github.com/nsi-iff/fluidity",
    "FormatBlock": "https://github.com/welbornprod/fmtblock",
    "freetype": "https://sourceforge.net/projects/indic-computing",
    "fs": "https://github.com/PyFilesystem/pyfilesystem2",
    "fsleyes-props": "https://git.fmrib.ox.ac.uk/fsl/fsleyes/props",
    "fsleyes-widgets": "https://git.fmrib.ox.ac.uk/fsl/fsleyes/widgets",
    "fs-s3fs": "https://github.com/PyFilesystem/s3fs",
    "fsspec": "https://github.com/intake/filesystem_spec",
    "ftputil": "https://ftputil.sschwarzer.net/",
    "galaxy-containers": "https://github.com/galaxyproject/galaxy",
    "galaxy-sequence-utils": "https://github.com/galaxyproject/sequence_utils",
    "galaxy-tool-util": "https://github.com/galaxyproject/galaxy",
    "galaxy-util": "https://github.com/galaxyproject/galaxy",
    "geoip": "https://github.com/maxmind/GeoIP-api-python",
    "geventhttpclient-wheels": "https://github.com/gwik/geventhttpclient",
    "gherkin-official": "https://github.com/cucumber/gherkin-python",
    "ghubunix": "https://github.com/BBloggsbott/ghub",
    "git-github-reflog": "https://github.com/criswell/github-reflog",
    "github-feedparser": "https://github.com/wadewilliams/github-atom-feed-parser",
    "gmpy2": "https://github.com/aleaxit/gmpy",
    "GnuPGInterface": "https://sourceforge.net/projects/py-gnupg",  # sf redirect
    "googleapis-common-protos": "https://github.com/googleapis/googleapis",
    "google-auth-httplib2": "https://github.com/GoogleCloudPlatform/google-auth-library-python-httplib2",
    "google-auth": "https://github.com/googleapis/google-auth-library-python",
    "google-auth-oauthlib": "https://github.com/GoogleCloudPlatform/google-auth-library-python-oauthlib",
    "googledatastore": "https://github.com/GoogleCloudPlatform/google-cloud-datastore",
    "google-endpoints-api-management": "https://github.com/cloudendpoints/endpoints-management-python",
    "google-endpoints": "https://github.com/cloudendpoints/endpoints-python",
    "google-gax": "https://github.com/googleapis/gax-python",
    "googlemaps": "https://github.com/googlemaps/google-maps-services-python",
    "google-python-cloud-debugger": "https://github.com/GoogleCloudPlatform/cloud-debug-python",
    "googletranslate": "https://github.com/weaming/py-googletrans",
    "gremlinpython": "https://github.com/apache/tinkerpop",
    "grpc-google-iam-v1": "https://github.com/googleapis/googleapis",
    "grpc-google-logging-v2": "https://github.com/googleapis/googleapis",
    "grpc-google-pubsub-v1": "https://github.com/googleapis/googleapis",
    "grpcio": "https://github.com/grpc/grpc",
    "guppy": "https://github.com/svenil/guppy-pe",
    "h2o": "https://github.com/h2oai/h2o-3",
    "h2o-pysparkling-2.1": "https://github.com/h2oai/sparkling-water",
    "h2o-pysparkling-2.2": "https://github.com/h2oai/sparkling-water",
    "h2o-pysparkling-2.3": "https://github.com/h2oai/sparkling-water",
    "h2o-pysparkling-2.4": "https://github.com/h2oai/sparkling-water",
    "hachoir-core": "https://bitbucket.org/haypo/hachoir",
    "hachoir-metadata": "https://bitbucket.org/haypo/hachoir",
    "hachoir-parser": "https://bitbucket.org/haypo/hachoir",
    "home-assistant-frontend": "https://github.com/home-assistant/home-assistant-polymer",
    "homeassistant": "https://github.com/home-assistant/core",
    "http-ece": "https://github.com/martinthomson/encrypted-content-encoding",
    "hubble-github": "https://github.com/jos-b/hubble",
    "hypothesis-pytest": "https://github.com/DRMacIver/hypothesis",  # old repo
    "ibm-db-sa": "https://github.com/ibmdb/python-ibmdb",
    "ibmiotf": "https://github.com/ibm-watson-iot/iot-python",
    "icinga2py": "https://github.com/tclh123/icinga2-api",
    "instana": "https://github.com/instana/python-sensor",
    "interpret-core": "https://github.com/interpretml/interpret",
    "intuit-oauth": "https://github.com/intuit/oauth-pythonclient",
    "ipmi": "https://github.com/zhao-ji/check_ipmi_sensor_v3",
    "isim": "https://github.com/dalemyers/xcrun",
    "j1m.sphinxautozconfig": "https://github.com/jimfulton/sphinxautozconfig",
    "javaobj-py3": "https://github.com/tcalmant/python-javaobj",
    "json-logic-qubit": "https://github.com/qubitproducts/json-logic-py",
    "jsonrpclib-pelix": "https://github.com/tcalmant/jsonrpclib",
    "jupyter-kernel-gateway": "https://github.com/jupyter-incubator/kernel_gateway",
    "jupyter-kernel-polymake": "https://github.com/sebasguts/jupyter-polymake",
    "jupyter-kernel-singular": "https://github.com/sebasguts/jupyter-singular",
    "kaggle": "https://github.com/Kaggle/kaggle-api",
    "keras2onnx": "https://github.com/onnx/keras-onnx",
    "kerberos": "https://github.com/apple/ccs-pykerberos",
    "kfp-server-api": "https://github.com/kubeflow/pipelines",
    "kivy-garden": "https://github.com/kivy-garden/garden",
    "kiwisolver": "https://github.com/nucleic/kiwi",
    "kubernetes": "https://github.com/kubernetes-client/python",
    "launchdarkly-server-sdk": "https://github.com/launchdarkly/python-server-sdk",
    "lark-parser": "https://github.com/erezsh/lark",
    "ldclient-py": "https://github.com/launchdarkly/python-server-sdk",
    "libarchive": "https://github.com/dsoprea/PyEasyArchive",
    "libclang-py3": "https://hg.anteru.net/python3-libclang",
    "libmagic": "https://bitbucket.org/xmonader/pymagic-dev",
    "librato-metrics": "https://github.com/librato/python-librato",
    "liccheck": "https://github.com/dhatim/python-license-check",
    "linaro-django-jsonfield": "https://launchpad.net/django-jsonfield",
    "linear-tsv": "https://github.com/solidsnack/tsv",
    "localstack-ext": "https://github.com/localstack/localstack",
    "log4tailer": "https://bitbucket.org/jordilin/alerta",
    "lxc-python2": "https://github.com/lxc/python2-lxc",
    "m3-cdecimal": "https://www.bytereef.org/mpdecimal",
    "magma-lang": "https://github.com/phanrahan/magma",
    "MapGitConfig": "https://github.com/zalando/python-gitconfig",
    "marbles.core": "https://github.com/twosigma/marbles",
    "marbles.mixins": "https://github.com/twosigma/marbles",
    "MDAnalysisTests": "https://github.com/mdanalysis/mdanalysis",
    "MDP": "https://sourceforge.net/projects/mdp-toolkit",
    "mdx_gh_links": "https://github.com/Python-Markdown/github-links",
    "medusa": "https://sourceforge.net/projects/oedipus",
    "mglob": "https://code.google.com/p/vvtools",
    "misspellings": "https://github.com/lyda/misspell-check",
    "mlperf-compliance": "https://github.com/mlperf/training",
    "monkeypatcher": "https://github.com/iki/monkeypatch",
    "msk": "https://github.com/MycroftAI/mycroft-skills-kit",
    "msm": "https://github.com/MycroftAI/mycroft-skills-manager",
    "msrest": "https://github.com/Azure/msrest-for-python",
    "murmurhash3": "https://github.com/veegee/mmh3",
    "mysql": "https://github.com/valhallasw/virtual-mysql-pypi-package",
    "mysql-python": "https://github.com/farcepest/MySQLdb1",
    "neovim": "https://github.com/neovim/python-client",
    "netstorageapi": "https://github.com/akamai/NetStorageKit-Python",
    "nflx-genie-client": "https://github.com/netflix/genie",
    "nimoy-framework": "https://github.com/browncoat-ninjas/nimoy",
    "nixio": "https://github.com/G-Node/nixpy",
    "node-semver": "https://github.com/podhmo/python-semver",
    "nose-parameterized": "https://github.com/wolever/parameterized",
    "numarray": "https://sourceforge.net/projects/numpy",
    "numpy-quaternion": "https://github.com/moble/quaternion",
    "oauth_provider": "http://code.welldev.org/django-oauth/",
    "oic": "https://github.com/OpenIDC/pyoidc",
    "onetoken": "https://github.com/1token-trade/onetoken-py-sdk",
    "opencensus-context": "https://github.com/census-instrumentation/opencensus-python",
    "opencensus-ext-azure": "https://github.com/census-instrumentation/opencensus-python",
    "opencensus-ext-grpc": "https://github.com/census-instrumentation/opencensus-python",
    "openexr": "http://www.excamera.com/sphinx/articles-openexr.html",
    "openshift": "https://github.com/openshift/openshift-restclient-python",
    "optimizely-sdk": "https://github.com/optimizely/python-sdk",
    "os_virtual_interfacesv2_python_novaclient_ext": "https://github.com/rackerlabs/os_virtual_interfacesv2_ext",
    "paperboy": "https://github.com/edgeflip/dispatch",
    "parameterizedtestcase": "https://github.com/msabramo/python_unittest_parameterized_test_case",
    "patch-ng": "https://github.com/conan-io/python-patch",
    "paypalrestsdk": "https://github.com/paypal/PayPal-Python-SDK",
    "pdbpp": "https://github.com/antocuni/pdb",
    "pdoc3": "https://github.com/pdoc3/pdoc",
    "perfume-bench": "https://github.com/leifwalsh/perfume",
    "phonenumberslite": "https://github.com/daviddrysdale/python-phonenumbers",
    "pip-ppm": "https://github.com/ekinertac/ppm",
    "pip-update": "https://github.com/gogobook/python-env-upgradable-packages-extractor",
    "pkgcheck-arch": "https://github.com/FFY00/pkgcheck",
    "plaidml-keras": "https://github.com/plaidml/plaidml",
    "poline": "https://github.com/riolet/pol",
    "precise-runner": "https://github.com/MycroftAI/mycroft-precise",
    "predicthq": "https://github.com/predicthq/sdk-py",
    "pretrainedmodels": "https://github.com/cadene/pretrained-models.pytorch",
    "prison": "https://github.com/betodealmeida/python-rison",
    "progressbar33": "https://github.com/germangh/python-progressbar",
    "proto-google-cloud-error-reporting-v1beta1": "https://github.com/googleapis/googleapis",
    "proto-google-cloud-logging-v2": "https://github.com/googleapis/googleapis",
    "proto-google-cloud-pubsub-v1": "https://github.com/googleapis/googleapis",
    "publicsuffix": "https://code.google.com/p/python-public-suffix-list",
    "pulsar-galaxy-lib": "https://github.com/galaxyproject/pulsar",
    "pulsectl": "https://github.com/mk-fg/python-pulse-control",
    "pure-protobuf": "https://github.com/eigenein/protobuf",
    "pur": "https://github.com/alanhamlett/pip-update-requirements",
    "pusher": "https://github.com/pusher/pusher-http-python",
    "pyaml": "https://github.com/mk-fg/pretty-yaml",
    "pybarcode": "https://bitbucket.org/whitie/python-barcode",
    "pybitbucket_fork": "https://github.com/guyzmo/pybitbucket",
    "pybitbucket": "https://bitbucket.org/atlassian/python-bitbucket",
    "pybloom": "https://github.com/jaybaird/python-bloomfilter",
    "pycocotools": "https://github.com/cocodataset/cocoapi",
    "pyculiar": "https://github.com/wdm0006/pyculiarity",
    "pydeck": "https://github.com/uber/deck.gl",
    "pydevd": "https://github.com/fabioz/PyDev.Debugger",
    "pydocumentdb": "https://github.com/Azure/azure-documentdb-python",
    "pydomo": "https://github.com/domoinc/domo-python-sdk",
    "pyfarmhash": "https://github.com/veelion/python-farmhash",
    "pyGRID": "https://github.com/OpenMined/PySyft",
    "pyinsane2": "https://github.com/openpaperwork/pyinsane",
    "pylama_gjslint": "https://github.com/klen/pylama",
    "pyload-ng": "https://github.com/pyload/pyload",
    "pymacaroons-pynacl": "https://github.com/matrix-org/pymacaroons",
    "pymatbridge": "https://github.com/arokem/python-matlab-bridge",
    "pynut2": "https://github.com/mezz64/python-nut2",
    "pyobjc-core": "https://github.com/ronaldoussoren/pyobjc",
    "pyopengl-accelerate": "https://sourceforge.net/projects/pyopengl",
    "pypakr-pkg": "https://github.com/ajanicij/pypakr",
    "pypd": "https://github.com/PagerDuty/pagerduty-api-python-client",
    "pypillowfight": "https://gitlab.gnome.org/World/OpenPaperwork/libpillowfight",
    "pyqt4": "https://www.riverbankcomputing.com/software/pyqt",
    "pyqt5": "https://www.riverbankcomputing.com/software/pyqt",
    "pyqt5-sip": "https://www.riverbankcomputing.com/software/sip",
    "pyros-genmsg": "https://github.com/ros/genmsg",
    "pyros-genpy": "https://github.com/ros/genpy",
    "PyRSS2Gen": "http://dalkescientific.com/Python/PyRSS2Gen.html",
    "pysnmp-mibs": "https://sourceforge.net/projects/pysnmp",
    "pytest-allure-adaptor": "https://github.com/allure-framework/allure-pytest",
    "pytest-fixture-config": "https://github.com/manahl/pytest-plugins",
    "pytest-git": "https://github.com/manahl/pytest-plugins",
    "pytest-listener": "https://github.com/manahl/pytest-plugins",
    "pytest-pyramid-server": "https://github.com/manahl/pytest-plugins",
    "pytest-reportportal": "https://github.com/reportportal/agent-python-pytest",
    "pytest-svn": "https://github.com/manahl/pytest-plugins",
    "pytest-verbose-parametrize": "https://github.com/manahl/pytest-plugins",
    "pytest-virtualenv": "https://github.com/manahl/pytest-plugins",
    "python-graph-core": "https://github.com/pmatiello/python-graph",
    "python-graph-dot": "https://github.com/pmatiello/python-graph",
    "python-owasp-zap-v2-4": "https://github.com/zaproxy/zap-api-python",
    "python-potr": "https://github.com/afflux/pure-python-otr",
    "pythonz-bd": "https://github.com/berdario/pythonz",
    "qdarkstyle": "https://github.com/ColinDuquesnoy/QDarkStyleSheet",
    "qiniu": "https://github.com/qiniu/python-sdk",
    "qt4reactor": "https://github.com/ghtdak/qtreactor",
    "rapid-framework": "https://github.com/BambooHR/rapid",
    "ravello-sdk": "https://github.com/ravello/python-sdk",
    "readability-lxml": "https://github.com/buriy/python-readability",
    "repoman-scm": "https://github.com/tuenti/python-repoman",
    "repostat": "https://github.com/JamesJeffryes/repo-stat-stash",
    "repoze.what.plugins.sql": "https://github.com/repoze/repoze.what-sql",
    "repoze.who.plugins.sa": "https://github.com/repoze/repoze.who-sqlalchemy",
    "requests-cache-latest": "https://github.com/reclosedev/requests-cache",
    "requests-opentracing": "https://github.com/opentracing-contrib/python-requests",
    "respite": "https://github.com/jgorset/django-respite",
    "rest_framework_auth0": "https://github.com/mcueto/djangorestframework-auth0",
    "robotremoteserver": "https://github.com/robotframework/PythonRemoteServer",
    "rules": "https://github.com/dfunckt/django-rules",
    "scspell3k": "https://github.com/myint/scspell",
    "sentry-sdk": "https://github.com/getsentry/sentry-python",
    "setuptools-lint": "https://github.com/johnnoone/setuptools-pylint",
    "sfctl": "https://github.com/Azure/service-fabric-cli",
    "sfmergeutility": "https://github.com/Azure/azure-cli-extensions",
    "sgmllib3k": "https://hg.hardcoded.net/sgmllib",
    "sidecar": "https://github.com/jupyter-widgets/jupyterlab-sidecar",
    "slackweb": "https://github.com/satoshi03/slack-python-webhook",
    "smdebug-rulesconfig": "https://github.com/awslabs/sagemaker-debugger-rulesconfig",
    "smdebug": "https://github.com/awslabs/sagemaker-debugger",
    "snowballstemmer": "https://github.com/snowballstem/snowball",
    "social-auth-app-django": "https://github.com/python-social-auth/social-app-django",
    "social-auth-app-flask": "https://github.com/python-social-auth/social-app-flask",
    "social-auth-app-flask-sqlalchemy": "https://github.com/python-social-auth/social-app-flask-sqlalchemy",
    "social-auth-core": "https://github.com/python-social-auth/social-core",
    "social-auth-storage-mongoengine": "https://github.com/python-social-auth/social-storage-mongoengine",
    "social-auth-storage-peewee": "https://github.com/python-social-auth/social-storage-peewee",
    "social-auth-storage-sqlalchemy": "https://github.com/python-social-auth/social-storage-sqlalchemy",
    "sparkdl": "https://github.com/allwefantasy/spark-deep-learning",
    "sq-blocks": "https://github.com/square/blocks",
    "sst": "https://launchpad.net/selenium-simple-test",
    "stormssh": "https://github.com/emre/storm",
    "stups-tokens": "https://github.com/zalando-stups/python-tokens",
    "stups-zign": "https://github.com/zalando-stups/zign",
    "subgrab": "https://github.com/RafayGhafoor/Subscene-Subtitle-Grabber",
    "suds-community": "https://github.com/suds-community/suds",
    "systemd-python": "https://github.com/systemd/python-systemd",
    "taskcluster-urls": "https://github.com/taskcluster/taskcluster-lib-urls",
    "tb-nightly": "https://github.com/tensorflow/tensorboard",
    "tdlib": "https://github.com/tdlib/td",
    "tester-coverage": "https://github.com/vb64/test.helper.coverage",
    "tg-ext-silverplate": "https://github.com/pedersen/tgtools",
    "tgmochikit": "https://sourceforge.net/projects/turbogears1",
    "tox-monorepo": "https://github.com/Azure/azure-sdk-tools",
    "transip": "https://github.com/benkonrath/transip-api",
    "trepan3k": "https://github.com/rocky/python3-trepan",
    "TurboCheetah": "https://sourceforge.net/projects/turbogears1",
    "TurboGears2": "https://github.com/Turbogears/tg2",
    "turbokid": "https://sourceforge.net/projects/turbogears1",
    "tvb-gdist": "https://github.com/the-virtual-brain/tvb-geodesic",
    "tweet-preprocessor": "https://github.com/s/preprocessor",
    "twitter.common.finagle-thrift": "https://github.com/twitter/commons",
    "twitter.common.lang": "https://github.com/twitter/commons",
    "twitter.common.rpc": "https://github.com/twitter/commons",
    "typing-extensions": "https://github.com/python/typing",
    "uamqp": "https://github.com/Azure/azure-uamqp-python",
    "uiautomation": "https://github.com/yinkaisheng/Python-UIAutomation-for-Windows",
    "ujson-bedframe": "https://github.com/esnme/ultrajson",
    "ujson": "https://github.com/ultrajson/ultrajson",
    "ujson-ia": "https://github.com/internetarchive/ultrajson",
    "ujson-x": "https://github.com/PawelTroka/ultrajson-x",
    "umalqurra": "https://github.com/tytkal/python-hijiri-ummalqura",
    "upyun": "https://github.com/upyun/python-sdk",
    "uszipcode": "https://github.com/MacHu-GWU/uszipcode-project",
    "utillib": "https://github.com/jor-/util",
    "vim-vint": "https://github.com/Kuniwak/vint",
    "vsts": "https://github.com/Microsoft/vsts-python-api",
    "weka-easypy": "https://github.com/weka-io/easypy",
    "wn": "https://github.com/nltk/wordnet",
    "woocommerce": "https://github.com/woocommerce/wc-api-python",
    "word2number": "https://github.com/akshaynagpal/w2n",
    "wordaxe": "https://sourceforge.net/projects/deco-cow",
    "ws4py": "https://github.com/Lawouach/WebSocket-for-Python",
    "XStatic-Angular-Schema-Form": "https://github.com/dmsimard/python-XStatic-Angular-Schema-Form-distgit",
    "XStatic-objectpath": "https://github.com/dmsimard/python-XStatic-objectpath-distgit",
    "XStatic-tv4": "https://github.com/dmsimard/python-XStatic-tv4-distgit",
    "yarn-api-client": "https://github.com/toidi/hadoop-yarn-api-python-client",
    "yolk3k": "https://github.com/myint/yolk",
    "youtube-dl-server": "https://github.com/jaimeMF/youtube-dl-api-server",
    "z3-solver": "https://github.com/Z3Prover/z3",
    "zabbix-api": "https://github.com/gescheit/scripts",
    "zc.recipe.egg": "https://github.com/buildout/buildout",
    "ZSI": "https://sourceforge.net/projects/pywebsvcs",
    "zvmcloudconnector": "https://github.com/mfcloud/python-zvm-sdk",
}

name_mismatches = mismatch = name_mismatch_metadata.copy()
mismatch.update(name_mismatch_fetched)

invalid = ["tf-nightly-2-0-preview"]

bad_metadata = [
    "2to3",
    "abclient",
    "affinity",
    "aiotask-context",
    "alooma",
    "Aman",
    "amazon-dax-client",
    "ana",
    "anaconda",
    "anitya",
    "anykeystore",
    "application",
    "aspects",
    "augeas",
    "augur",
    "awsebcli",
    "awshelpers",
    "azure-functions",
    "azureml-automl-core",
    "azureml-automl-runtime",
    "azureml-core",
    "azureml-dataprep-native",
    "azureml-dataprep",
    "azureml-defaults",
    "azureml-explain-model",
    "azureml-interpret",
    "azureml-mlflow",
    "azureml-pipeline-core",
    "azureml-pipeline-steps",
    "azureml-pipeline",
    "azureml-sdk",
    "azureml-telemetry",
    "azureml-train-automl-client",
    "azureml-train-automl-runtime",
    "azureml-train-automl",
    "azureml-train-core",
    "azureml-train",
    "azureml-train-restclients-hyperdrive",
    "azureml-widgets",
    "bencode",
    "bigsuds",
    "bindep",
    "BitBucket",
    "bitbucket-cli",
    "BitVector",
    "bobodoctestumentation",
    "bottle-sqlite",
    "bs4",
    "btrc",
    "bugjar",
    "bugsnag",
    "Camelot",
    "carton",
    "catkin-sphinx",
    "cd",
    "cerealizer",
    "Cheesecake",  # should be https://github.com/griggheo/cheesecake
    "chromedriver",
    "chubby",
    "chubby",
    "client",
    "cloudshell-automation-api",
    "cmsis-pack-manager",
    "cognite-logger",
    "comet-git-pure",
    "comet-ml",
    "config",
    "coverage_pth",
    "cronwatch",
    "cvxopt",
    "daemon",
    "darcsver",  # add https://tahoe-lafs.org/trac/darcsver
    "databricks",
    "databricks-connect",  # https://github.com/Menziess/databricks-connect
    "databricks-pypi",
    "databricks-pypi1",
    "databricks-pypi2",
    "databricks-pypi-extras",
    "dateutils",  # todo
    "dbf",
    "dbus-python",
    "dci-utils",
    "demjson",
    "dependency_management",  # recheck
    "dai-sgqlc-3-5",
    "dialog",
    "digitalocean",
    "distribute",
    "distutils",
    "divide-and-cover",
    "django-dajaxice",  # https://github.com/jorgebastida/django-dajaxice/issues/158
    "django-datagrid",  # same company produces https://github.com/agiliq/Django-parsley, sometimes found
    "django-hijack",
    "django-inline-ordering",  # uses 'https://code.google.com/p/django-grappelli'
    "django-lfstheme",  # should be 'https://github.com/diefenbach/lfs-theme'
    "django-myrecaptcha",  # https://bitbucket.org/Kizlum/django-myrecaptcha ; https://bitbucket.org/pelletier/django-myrecaptcha exists still
    "django-settings-toml",
    "django-staticmediamgr",
    "django-toolbelt",  # finds https://github.com/heroku/django-heroku
    "djtracker",
    "dmidecode",
    "dm.xmlsec-binding",  # no repo exists by https://github.com/d-maurer, https://github.com/ansible/dm.xmlsec.binding is best attempt
    "dns",
    "dogstatsd-python",
    "dopen",
    "DoubleMetaphone",
    "dox",
    "dox-mortar",
    "dtopt",  # https://svn.colorstudy.com/dtopt offline
    "Durus",
    "electrical-calendar",
    "encutils",  # Should be https://code.google.com/p/cssutils
    "enum",  # stdlib
    "ephemeral-port-reserve",
    "epl",
    "evtx",
    "fiat",
    "fig",
    "firkin",
    "flojay",
    "fpconst",
    # "galaxy" very slow https://github.com/jayvdb/pypidb/issues/85
    "gen3git",
    "getch",
    "github3py",
    "github-issues-tools",
    "github-release",
    "Gloo",
    "google",
    "google-cloud-dataflow",
    "graph-tool",  # https://github.com/count0/graph-tool
    "hurry-filesize",
    "hypothesis-pytest",  # now part of https://github.com/DRMacIver/hypothesis
    "IGitt",
    "imblearn",
    "inifile",
    "intel-openmp",  # lots of odd results
    "interfile",
    "interval",
    "ipympl",
    "ipysheet",
    "jieba3k",  # py3 branch
    "jupyterlab-pygments",
    "justbases",
    "kaa-base",
    "kaa-metadata",
    "keystoneclient",
    "kid",
    "kitchen",
    "kmod",
    "launchpad-nag",
    "lightstep",
    "locust",
    "logentries",
    "logger",
    "lpshipit",
    "ly",
    "mailgun",
    "mailman-web",  # git@gitlab.com:mailman/mailman-web in PyPI description not detected
    "mallard-ducktype",
    "marionette",
    "marshmallow-enum",
    "membrete",
    "microversion-parse",
    "mkl",  # intel proprietary
    "moksha-common",
    "moksha-hub",
    "mongoalchemy",
    "monthdelta",
    "Morfessor",  # needs scraping depth 2 to find https://github.com/aalto-speech/morfessor mentioned on http://morpho.aalto.fi/projects/morpho/morfessor2.html
    "mozterm",
    "msal-extensions",
    "multicoder",
    "multimethods",
    "myghty",  # replaced with mako; homepage redirects back to pypi
    "mysql-connector",
    "mysql-connector-python",
    "mysql-connector-python-rf",
    "neptune-client",  # Finds https://github.com/neptune-ml/neptune-contrib
    "netapp-lib",
    "newrelic",
    "nose-fixes",
    "nose-fixes",  # < 10 downloads per day
    "nose-xunitmp",
    "np-utils",
    "nvidia-ml-py",
    "nvidia-ml-py3",
    "onedrivesdk",
    "openoffice",
    "openopt",
    "OrangeAssassin",
    "ordereddict",  # TODO: give it an activestate url
    "os-diskconfig-python-novaclient-ext",
    "oslo-sphinx",
    "overtest",  # redis results skipped
    "ovirt-engine-sdk",
    "ovirt-engine-sdk-python",
    "p4python",
    "packit",
    "pam",
    "panda",
    "password",
    "performance",
    "petlink",
    "pigpio",
    "pipit",  # https://github.com/youngking/pipit
    "pinax",
    "pip-reqs",
    "Pivy",
    "PlanOut",
    "pomegranate",
    "pprint",
    "prctl",
    "PrintDebug",
    "PsyLab",
    "ptrace",
    "pubnub",  # todo
    "py1cmd",
    "pyalsa",  # http://cvs.bigasterisk.com/viewcvs/pyalsa/ offline
    "pybee",
    "pybibframe",
    "pychart",
    "pycmd",
    "pycpuinfo",
    "pyfim",
    "pyformance",
    "pygal",
    "pygments-ansi-color",
    "pyjwkest",
    "pykdtree",
    "pykerberos",
    "pylirc",
    "pylogo",  # https://svn.colorstudy.com/PyLogo offline
    "PyMOL",
    "pymol",  # https://github.com/schrodinger/pymol-open-source, not on PyPI
    "pypiwin32",
    "pyprimes",
    "PyPrint",
    "pyprocdev",
    "PyProtocols",  # http://svn.eby-sarna.com/PyProtocols/, now at https://github.com/PEAK-Legacy/PyProtocols
    "Pyro",
    "pyrpm",
    "pyside2",
    "pyside2",
    "pyspark-dist-explore",
    "pytest-coverage",
    "pytext",
    "python-axolotl-curve25519",
    "python-docs-theme",
    "python-graph",  # no files, https://github.com/Shoobx/python-graph/blob/master/core/setup.py
    "python-hglib",
    "python-igraph",  # finds https://github.com/pygobject/pycairo
    "python-mpd",
    "python-uinput",
    "pythonwhois",
    "pytictoc",
    "pytidylib",
    "pytorch",
    "pyudorandom",
    "pyVirtualize",
    "pyweblib",
    "pyzipcode",
    "qet_tb_generator",
    "quickproxy",
    "random2",
    "reddit",
    "replit",  # todo
    "reportlab",
    "requestsexceptions",
    "ReviewBoard",
    "rfc6555",  # 0.0.0 (fixme)
    "robinhood-aiokafka",
    "rook",
    "ruptures",
    "s3pip",
    "s3pypi",
    "saganvm",
    "salt",
    "sane",
    "schevodurus",  # https://github.com/11craft/schevogears
    "scm-git",
    "score.projects",
    "script",
    "sed",
    "selection",
    "setuptools_cython",
    "shiboken2",
    "shippo",
    "simplewrap",
    "singer-python",
    "skimage",
    "sklearn",
    "SmartlingApiSdk",
    "solartime",
    "spark",  # www.hugedomains.com (http://trac.pytan.com)
    "sphinxcontrib-infrae",
    "sq-native",
    "sqlalchemy-teradata",
    "stacks",
    "staff-graded-xblock",
    "stemming",  # todo
    "strict-rfc3339",
    "stscipython",
    "suds",
    "supercaptcha",
    "tableauhyperapi",
    "tambo",
    "Tempita",
    "tensorboard-plugin-wit",
    "tensorflow-addons",
    "termcolor",
    "testflo",
    "testinfiniteloop",
    "tgfastdata",
    "timedop",
    "toastcord",
    "ToscaWidgets",
    "tracer",
    "travis",
    "triforce",
    "troveclient",
    "try",
    "ts-flint",
    "tw.dynforms",
    "tw.forms",
    "typing",
    "typogrify-engineer",
    "unittest",
    "urlopen",
    "urlparse3",
    "usbinfo",
    "vigilance",
    "viper",  # 2011, governance
    "volatility",
    "waiting",
    "watson-machine-learning-client",  # todo
    "wave",
    "webpy",  # joe job
    "websocket",
    "webunit",
    "widgetsnbextension",
    "Willow",
    "wmi",
    "xmltramp",  # cert failure & no files
    "zbase32",
    "zmq",
    "zuul-sphinx",  # https://opendev.org/zuul/zuul-sphinx choosing https://opendev.org/zuul/zuul instead, or None
]

sometimes_wrong_result = [
    "pyfeyn",  # https://svn.hepforge.org/pyfeyn and hepforge.org is offline
]

wrong_result = []

missing_repos = [
    "aioeventlet",
    "alfredcmd",  # https://github.com/GustavoKatel/alfredcmd
    "azureml-model-management-sdk",  # https://github.com/Azure/azure-ml-api-sdk-python
    "baruwa",  # https://github.com/akissa/baruwa disappeared
    "bcdoc",
    "boletin",  # https://tracpub.yaco.es/djangoapps/wiki/Boletin
    "clint2",  # https://github.com/myusuf3/clint
    "coding",  # https://bitbucket.org/krp/coding, http://code.activestate.com/pypm/coding/  by K Richard Pixley
    "curl2requests",  # https://github.com/a4m/curl2requests
    "datarobot",  # https://github.com/datarobot/public_api_client
    "dedupe-variable-datetime",  # https://github.com/datamade/dedupe-variable-datetime
    "debian",
    "django-attention",  # https://bitbucket.org/zacharyvoase/django-attention disappeared
    "django-cpserver",  # https://hg.piranha.org.ua/cpserver archived at https://github.com/lincolnloop/django-cpserver ?
    "django-debug",  # https://bitbucket.org/lorien/django-debug disappeaed
    "django-genericforeignkey",  # https://tracpub.yaco.es/..
    "django-livevalidation",
    "django-oot",  # https://tracpub.yaco.es/..
    "django-pingback",  # https://hg.piranha.org.ua/django-pingback
    "django-quickedit",
    "django-reporter",
    "django-retracer",
    "django-simple-cart",  # https://bitbucket.org/lorien/django-simple-cart
    "django-simple-news",
    "django-sproutcore",  # lost repo? https://github.com/sproutcoreisyournewbicycle/django-spoutcore has no setup.py check it even has a setup.py
    "django-stdfile",  # https://tracpub.yaco.es/djangoapps/wiki/StdFile
    "dnsproxy",  # https://github.com/blurrcat/dnsproxy
    "docker-buildtool",  # https://github.com/openai/docker-buildtool
    "elftools",  # https://github.com/jme2041/elftools
    "emencia_django_newsletter",
    "flask-mail",
    "fontconfig",  # https://bitbucket.org/gm/fontconfig
    "galley",  # https://github.com/rackerlabs/galley
    "github-console",  # https://github.com/luoyeah/github-console
    "github-languages",  # https://github.com/upcwangying/github-languages-python
    "github-repo",  # https://github.com/looking-for-a-job/github-repo.py (user disappeared)
    "glob3",
    "hachoir-core",  # haypo
    "hachoir-metadata",  # haypo
    "hachoir-parser",  # haypo
    "hgdistver",  # https://bitbucket.org/RonnyPfannschmidt/hgdistver decommissioned due to setuptools_scm
    "http-client",  # https://github.com/huynhminhson/http_client
    "ipmi",
    "iptcinfo",  # https://bitbucket.org/gthomas/iptcinfo , history retained at https://github.com/20minutes/iptcinfo
    "kubetop",  # https://github.com/leastauthority.com/kubetop
    "logging",
    "magic",  # is a DreamHost 404 page
    "metafone",  # https://github.com/al45tair/metaphone, but this is a dead fork
    "nose-testconfig",  # https://bitbucket.org/jnoller/nose-testconfig history at https://github.com/singingwolfboy/nose-testconfig
    "octohot",  # https://github.com/Hotmart-Org/octohot
    "pdfpages",
    "pip-extras",  # https://github.com/russel/pip_extras
    "pip-upgrade",  # https://github.com/alistair-broomhead/pip_upgrade
    "pybarcode",  # https://bitbucket.org/whitie/python-barcode/ history preserved at https://github.com/emfcamp/python-barcode
    "pygradle",  # https://github.com/mwalkowski/pygradle
    "pypi-uploader",  # https://github.com/ignacysokolowski/pypi-uploader
    "pyrepo",  # https://github.com/dghubble/pyrepo
    "python3-xlib",  # https://github.com/LiuLang/python3-xlib
    "python-emailahoy3",  # https://github.com/febrezo/python-emailahoy3
    "python-json2yaml",  # https://github.com/appstore-zencore/python-json2yaml
    "pythonloc",  # https://github.com/pipxproject/pythonloc
    "pythonpy",  # https://github.com/Russell91/pythonpy
    "rxjson",  # https://github.com/spiral-project/rxjson
    "safe-cast",  # https://github.com/jefftune/safe-cast
    "Schevo",
    "schevodurus",
    "SchevoGears",
    "SchevoSql",
    "searchform",  # https://tracpub.yaco.es/djangoapps/wiki/SearchForm
    "sgmllib3k",  # https://hg.hardcoded.net/sgmllib
    "spotinst-agent",  # https://github.com/spotinst/spotinst-sdk-python/issues/65
    "structurizr",  # https://github.com/sixty-north/structurizr
    "svg",  # https://github.com/007gzs/svg dummy package
    "telepathy",  # https://github.com/hackliff/intuition-plugins
    "tempdir",  # https://bitbucket.org/another_thomas/tempdir
    "transhette",  # https://tracpub.yaco.es/djangoapps/wiki/Transhette
    "unboxftpd",  # rewritten by same author to be 'django-ftpserver'
    "update",
    "virtualenvwrapper-django",  # https://bitbucket.org/dhellmann/virtualenvwrapper.django see also https://github.com/tokibito/djangohosting/commit/d07f28d57b753304479c18666795cbd4c3be3bd0 ; history preserved at https://github.com/dgladkov/virtualenvwrapper.django
    "xunitparser",  # https://git.p.engu.in/laurentb/xunitparser
]

setuppy_mismatches = [
    "acme",  # part of certbot
    "adduserpath",
    "aliyun-",
    "allure-",
    "ansible-tower-cli",
    "antlr-",
    "antlr3-",
    "antlr4-",
    "apache-beam",
    "airtable-python-wrapper",
    "asgi-redis",
    "aspy.yaml",
    "aws-cdk-",
    "awscli-cwlogs",
    "aws-kinesis-agg",
    "azure-cosmosdb-nspkg",
    "bert-serving-client",
    "bert-serving-server",
    "bert-tensorflow",
    "better-exceptions-fork",
    "bigcode-",
    "blurb",
    "boto3-type-annotations",
    "boto3-type-annotations-with-docs",
    "brewer2mpl",  # github rename
    "buildbot-",
    "cangjie",  # fork of https://pypi.org/project/XKT/
    "catkin-pkg",
    "Catwalk",
    "certbot-",
    "certbot$",
    "chart-studio",
    "cheetah",
    "cl",
    "compatibility-lib",
    "cronwatch",
    "cryptography-vectors",
    "cupy-cuda100",
    "dataclasses-serialization",
    "datadog",
    "dbt-",
    "devpi-",
    "django-announcements",
    "django-autoslug",  # lots o forks
    "django-celery-email",
    "django-logicaldelete",  # pinax-models
    "django-notification",
    "django-pyodbc-azure",  # wrong branch
    "django-ranged-response",  # -> django-ranged-fileresponse
    "djangorestframework-jwt",
    "django-rest-polymorphic",
    "django-storages-redux",
    "doc-warden",  # azure
    "dotnetcore2",  # doesnt contain the Python wrapper
    "dnspython3",  # dnspython
    "drf-jwt",  # lots o forks
    "ebcdic",
    "elasticsearch2",
    "elasticsearch5",
    "elasticsearch6",  # same as elasticsearch
    "eyes-selenium",
    "facebookads",
    "fake-factory",  # renamed to faker
    "fastdiff",  # python/ subdir
    "fbprophet",
    "feather-format",
    "flake8-junit-report",
    "fluent",
    "freecell-solver",  # Python may not in https://github.com/shlomif/fc-solve
    "fysom",
    "galaxy-",
    "gapic-",
    "gax-",
    "gcloud-",
    "gcloud$",
    "GeoFinder",  # 'Setup.py' vs 'setup.py'
    "geventhttpclient-wheels",
    "girder-client",  # renamed to girder
    "gitdb2",  # 'mirror' of gitdb (i.e. they rolled it back into the old project, and maybe the repos should actually be different)
    "google-api-",
    "googleapis-",
    "googleappenginecloudstorageclient",
    "google-cloud-",
    "google-cloud$",
    "google-compute-",
    "googledatastore",
    "google-nucleus",
    "google-python-cloud-debugger",  # src/
    "googletranslate",  # wrong name in setup.py
    "graphql-core-next",  # old repo for graphql-core
    "gremlinpython",
    "grpc-google-",
    "grpcio-",
    "grpcio$",
    "h2o",
    "h2o-",
    "httpcore",  # renamed to httpx
    "http-ece",
    "husl",  # https://github.com/hsluv/hsluv-python/blob/master/setup.py uses hsluv, but hsluv is 404 on pypi
    "hypothesis",
    "ibm-db",
    "ibm-db-sa",
    "ibmiotf",
    "igraph",  # renamed to jgraph
    "interpret-community",
    "interpret-core",
    "jinja",
    "jinja2",  # https://github.com/pallets/jinja2-compat
    "jupyter-nbformat",
    "jupyterlab-launcher",  # redirects to jupyterlab_server
    "keras2onnx",
    "kfp-server-api",
    "kivy-garden",  # https://github.com/kivy-garden/garden/pull/34
    "ldclient-py",
    "letsencrypt",
    "lit",
    "localstack-ext",
    "mapgitconfig",  # this is now https://gitlab.com/jmcs/python-gitconfig/blob/master/setup.py , which does include Map
    "MDAnalysis",  # setup not in root
    "MDAnalysisTests",  # lives in MDAnalysis test suite
    "mlperf-compliance",
    "monkeypatcher",  # -> monkeypatch
    "moretools",
    "mwlib-",
    "mxnet",
    "mysqlclient",
    "neo4j-driver",
    "nflx-genie-client",
    "nimbusml",
    "nose-parameterized",
    "nox-automation",
    "numpy-stl",
    "open3d-python",  # Intel
    "opencensus-ext-",
    "opentelemetry-api",
    "pantsbuild-pants",
    "patch",
    "path-py",  # -> path
    "pattern3",
    "phonenumberslite",  # -> phonenumbers
    "pinax-notifications",
    "pip-requ",
    "plaidml",
    "ply",  # has no setup.py
    "proto-",
    "proto-google-",
    "pyarrow",
    "pybitbucket-fork",
    "pybuilder-pip-tools",
    "pycli",  # name lives in <src>/__init__.py
    "pycocotools",
    "pydeck",
    "pydevd-pycharm",
    "pydocumentdb",  # repo & project renamed to azure-cosmos
    "pydot2",  # folded back into pydot project
    "PyGrid",
    "pyinsane2",
    "pylama-gjslint",
    "pymorphy2-dicts",
    "pyobjc-",
    "pyopenssl",
    "pypic",
    "pyros-",  # stringops to create name
    "pyshark",  # setup in src
    "pyspark",
    "pysyft",
    "pytest-codestyle",  # renamed to pytest-pycodestyle
    "pytest-devpi-server",
    "pytest-fixture-config",
    "pytest-git",
    "pytest-listener",
    "pytest-profiling",
    "pytest-pyramid-server",
    "pytest-qt-app",
    "pytest-server-fixtures",
    "pytest-shutil",
    "pytest-svn",
    "pytest-verbose-parametrize",
    "pytest-virtualenv",
    "python-awk",
    "python-geoip-geolite2",
    "python-graph-core",
    "python-graph-dot",
    "python-owasp-zap-v2-4",
    "pythonz-bd",  # setup.py contains pythonz
    "pytorch-pretrained-bert",
    "pytorch-transformers",
    "pyuwsgi",
    "py-vapid",
    "pyviz",
    "pywatchman",
    "Qprompt",
    "rasa-nlu",  # merged into rasa
    "ray",
    "records",
    "referer-parser",
    "requests-cache-latest",  # copy of requests-cache
    "rfpimp",  # src dir
    "rosdistro",
    "rpm",
    "rwt",  # pip-run github rename
    "satyr",  # mentor
    "scikits-sparse",
    "scons",
    "sfctl",
    "sfmergeutility",
    "sfx-jaeger-client",
    "shadowsocks",  # https://github.com/shadowsocks/shadowsocks "removed according to regulations"
    "sk-video",
    "smmap2",  # 'mirror' of smmap
    "smsapi-client",
    "snowballstemmer",
    "sparkdl",
    "sseclient-py",
    "stompest",
    "stompest",
    "tb-nightly",
    "tdlib",
    "tensorflow-",
    "teradatasql",
    "tf-estimator-nightly",
    "tf-nightly",
    "tg-ext-silverplate",  # empty repo
    "torchgeometry",
    "tox-monorepo",
    "treeherder-client",
    "trollius",  # branch trollius
    "turbomail",
    "turicreate",  # src/python/
    "tvb-gdist",  # string is split in setup.py
    "twitter-common-",
    "typing-extensions",
    "ujson-",  # lots of forks
    "umalqurra",
    "uritemplate-py",
    "vsts",
    "vsts-",
    "webapp",
    "XStatic-Angular-Schema-Form",
    "XStatic-objectpath",
    "xstatic-term-js",
    "XStatic-tv4",
    "yowsup2",  # repo contains yowsup
    "z3-solver",  # src/api/python/setup.py
    "zabbix-api",
    "zetup",
    "zodb3",
]

setuppy_mismatches_xstatic = [
    "xstatic-angular-lrdragndrop",
    "xstatic-bootstrap-datepicker",
    "xstatic-datatables",
    "xstatic-hogan",
    "xstatic-jquery",
    "xstatic-jquery-ui",
    "xstatic-jquery-migrate",  # these use https://github.com/openstack/xstatic-angular-lrdragndrop/blob/master/xstatic/pkg/angular_lrdragndrop/__init__.py
    "xstatic-jquery-quicksearch",
    "xstatic-jquery-tablesorter",
    "xstatic-magic-search",
    "xstatic-rickshaw",
    "xstatic-smart-table",
    "xstatic-spin",
]

expected = {}
expected.update(exact)
expected.update(mismatch)

failures = bad_metadata + sometimes_wrong_result + wrong_result + missing_repos
